@@ -826,6 +826,9 @@ class Instruction:
         if self.opcode == Opcodes.DUP_TOP_TWO:
             return 2, 4, None
 
+        if self.opcode == Opcodes.ROT_THREE:
+            return 3, 3, None
+
         raise RuntimeError(self)
 
     def special_stack_affect_when_followed_by(self, instr: "Instruction") -> int:
@@ -1527,6 +1530,13 @@ class MutableFunction:
             if not (0 <= arg <= 255 and 0 <= instruction.opcode <= 255):
                 print("error", instruction)
 
+                print("----")
+
+                for ins in instructions:
+                    print(repr(ins))
+
+                print("----")
+
             self.__raw_code += bytes([instruction.opcode, arg])
 
     def get_raw_code(self):
