@@ -131,7 +131,22 @@ class MutableFunctionWithTree:
             self.print_recursive(root.arg_value, visited, level + 1)
 
 
-def prefix_all_locals_with(
+def prefix_all_locals_with_all(
+    mutable: MutableFunction | MutableFunctionWithTree,
+    prefix: str,
+):
+    if isinstance(mutable, MutableFunctionWithTree):
+        mutable.mutable.assemble_instructions_from_tree(mutable.root)
+        mutable = mutable.mutable
+
+    mutable.shared_variable_names = [prefix + e for e in mutable.shared_variable_names]
+
+    for instruction in mutable.instructions:
+        if instruction.has_local():
+            instruction.change_arg(instruction.arg)
+
+
+def prefix_all_locals_with_specified(
     mutable: MutableFunction | MutableFunctionWithTree,
     prefix: str,
     protected_locals: typing.List[str] = tuple(),
@@ -145,8 +160,6 @@ def prefix_all_locals_with(
         for e in mutable.shared_variable_names
     ]
 
-    print(mutable.shared_variable_names)
-
     for instruction in mutable.instructions:
         if instruction.has_local():
             instruction.update_owner(mutable, instruction.offset)
@@ -266,6 +279,7 @@ def insert_method_into(
     offset: typing.Union[Instruction, int],
     to_insert: MutableFunction | MutableFunctionWithTree,
     protected_locals: typing.List[str] = tuple(),
+    drop_return_result=True,
 ):
     """
     Inserts the function AFTER the given offset / Instruction.
@@ -293,7 +307,13 @@ def insert_method_into(
     for instr in to_insert.instructions:
         instr.offset = -1
 
-    prefix_all_locals_with(to_insert, to_insert.function_name + ":", protected_locals)
+    if protected_locals is not None:
+        prefix_all_locals_with_specified(
+            to_insert, to_insert.function_name + ":", protected_locals
+        )
+    else:
+        prefix_all_locals_with_all(to_insert, to_insert.function_name + ":")
+
     replace_opcode_with_other(
         to_insert, Opcodes.RETURN_VALUE, Opcodes.INTERMEDIATE_INNER_RETURN
     )
@@ -309,12 +329,12 @@ def insert_method_into(
             previous.next_instruction = instr
 
         if instr.opcode == Opcodes.INTERMEDIATE_INNER_RETURN:
-            previous.next_instruction = Instruction.create(Opcodes.POP_TOP)
-            previous.next_instruction.update_owner(to_insert, -1)
-            previous.next_instruction.next_instruction = instr
+            if drop_return_result:
+                previous.insert_after(Instruction(to_insert, -1, Opcodes.POP_TOP))
 
-            instr.change_opcode(Opcodes.JUMP_ABSOLUTE)
-            instr.change_arg_value(HEAD_INSTRUCTION.next_instruction)
+            instr.change_opcode(
+                Opcodes.JUMP_ABSOLUTE, HEAD_INSTRUCTION.next_instruction
+            )
 
         previous = instr
 
@@ -380,3 +400,30 @@ def insert_method_into(
     body.mutable.assemble_instructions_from_tree(body.root)
 
     # body.print_recursive()
+
+
+def inline_calls_to_const_functions(mutable: MutableFunction):
+    from bytecodemanipulation.Optimiser import _OptimisationContainer
+
+    for instr in mutable.instructions[:]:
+        if instr.opcode == Opcodes.CALL_FUNCTION:
+            source = next(instr.trace_stack_position(instr.arg))
+
+            if source.opcode != Opcodes.LOAD_CONST:
+                continue
+
+            target = source.arg_value
+
+            container = _OptimisationContainer.get_for_target(target)
+            if not container.try_inline_calls:
+                continue
+
+            instr.change_opcode(Opcodes.NOP)
+            insert_method_into(
+                mutable,
+                instr.offset,
+                MutableFunction(target),
+                drop_return_result=False,
+                protected_locals=None,
+            )
+            source.change_opcode(Opcodes.NOP)
