@@ -1,6 +1,11 @@
 import typing
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import AbstractSourceExpression
+from bytecodemanipulation.assembler.AbstractBase import (
+    IIdentifierAccessor,
+    MacroExpandedIdentifier,
+    StaticIdentifier,
+)
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.syntax_errors import _syntax_wrapper
 from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
@@ -53,8 +58,7 @@ from bytecodemanipulation.data.shared.expressions.SubscriptionAccessExpression i
 from bytecodemanipulation.data.shared.expressions.TopOfStackAccessExpression import (
     TopOfStackAccessExpression,
 )
-
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.data.shared.instructions.OpAssembly import AbstractOpAssembly
 
 from bytecodemanipulation.MutableFunction import Instruction
 
@@ -184,7 +188,7 @@ class Parser(AbstractParser):
 
         if namespace_part:
             if isinstance(namespace_part, str):
-                if self.scope.scope_path.pop() != namespace_part:
+                if self.scope.scope_path.pop(-1) != namespace_part:
                     raise RuntimeError
             else:
                 if self.scope.scope_path[-len(namespace_part) :] != namespace_part:
@@ -281,7 +285,7 @@ class Parser(AbstractParser):
             ).assemblies:
                 if macro.allow_assembly_instr:
                     self.rollback()
-                    print(name)
+                    # print(name)
                     return AbstractCallAssembly.IMPLEMENTATION.consume_macro_call(
                         self, scope
                     )
@@ -308,6 +312,7 @@ class Parser(AbstractParser):
         :param allow_op: if operations are allowed, starting with OP
         :param allow_advanced_access: if expressions like @global[$index].attribute are allowed
         :param scope: the parsing scope instance
+        :param allow_calls: if True, calls will be allowed as expressions
         """
         start_token = self.try_inspect()
 
@@ -326,6 +331,14 @@ class Parser(AbstractParser):
                     integer,
                 )
 
+            if isinstance(start_token, IdentifierToken):
+                if start_token.text == "None":
+                    return ConstantAccessExpression(None, start_token)
+                elif start_token.text == "True":
+                    return ConstantAccessExpression(True, start_token)
+                elif start_token.text == "False":
+                    return ConstantAccessExpression(False, start_token)
+
         if not isinstance(start_token, (SpecialToken, IdentifierToken)):
             return
 
@@ -357,7 +370,13 @@ class Parser(AbstractParser):
 
         elif start_token.text == "%" and allow_tos:
             self.consume(SpecialToken("%"), err_arg=scope)
-            expr = TopOfStackAccessExpression()
+
+            offset = self.try_consume(IntegerToken)
+
+            if offset is not None:
+                return TopOfStackAccessExpression(start_token, int(offset.text))
+
+            expr = TopOfStackAccessExpression(start_token)
 
         elif start_token.text == "~":
             self.consume(SpecialToken("~"), err_arg=scope)
@@ -369,11 +388,29 @@ class Parser(AbstractParser):
             self.consume(SpecialToken("\\"), err_arg=scope)
             expr = DiscardValueExpression()
 
-        elif start_token.text == "OP" and allow_op and "OP" in self.INSTRUCTIONS:
-            self.consume(start_token)
-            self.consume(SpecialToken("("), err_arg=scope)
-            expr = self.INSTRUCTIONS["OP"].consume(self, scope)
-            self.consume(SpecialToken(")"), err_arg=scope)
+        elif (
+            start_token.text == "OP"
+            and allow_op
+            and "OP" in self.INSTRUCTIONS
+            and AbstractOpAssembly.IMPLEMENTATION is not None
+        ):
+            self.consume(start_token, err_arg=scope)
+
+            if not (opening := self.try_consume(SpecialToken("("))):
+                raise throw_positioned_syntax_error(
+                    scope,
+                    self[-1:1],
+                    "expected '(' after OP when used in expressions",
+                )
+
+            expr = AbstractOpAssembly.IMPLEMENTATION.consume(self, scope)
+
+            if not self.try_consume(SpecialToken(")")):
+                raise throw_positioned_syntax_error(
+                    scope,
+                    [opening, self[0]],
+                    "expected ')' after operation",
+                )
 
         else:
             return
@@ -412,13 +449,14 @@ class Parser(AbstractParser):
                     )
 
                 elif self.try_consume(SpecialToken(".")):
-                    if self.try_consume(SpecialToken("(")):
+                    if opening_bracket := self.try_consume(SpecialToken("(")):
                         if not (
                             index := self.try_consume_access_to_value(
                                 allow_primitives=True,
                                 allow_tos=allow_tos,
                                 allow_op=allow_op,
                                 scope=scope,
+                                allow_calls=allow_calls,
                             )
                         ):
                             raise throw_positioned_syntax_error(
@@ -436,15 +474,19 @@ class Parser(AbstractParser):
 
                         if not self.try_consume(SpecialToken(")")):
                             raise throw_positioned_syntax_error(
-                                scope, self.try_inspect() or self[-1], "expected ')'"
+                                scope,
+                                [opening_bracket, self.try_inspect()],
+                                "expected ')'",
                             )
 
                         expr = DynamicAttributeAccessExpression(expr, index)
+
                     elif self.try_consume(SpecialToken("!")):
-                        name = self.try_consume(IdentifierToken)
+                        name = self.parse_identifier_like(scope)
                         expr = StaticAttributeAccessExpression(expr, name)
+
                     else:
-                        name = self.try_consume(IdentifierToken)
+                        name = self.parse_identifier_like(scope)
                         expr = AttributeAccessExpression(expr, name)
 
                 elif self.try_inspect() == SpecialToken("(") and allow_calls:
@@ -500,57 +542,14 @@ class Parser(AbstractParser):
 
         self.rollback()
 
-    def try_parse_identifier_like(self) -> typing.Callable[[ParsingScope], str] | None:
+    def try_parse_identifier_like(self) -> IIdentifierAccessor | None:
         if expr := self.try_consume_multi([SpecialToken("&"), IdentifierToken]):
-
-            def get(scope: ParsingScope):
-                # If this is None, we are only inspecting
-                if scope is None:
-                    return str(get)
-
-                if expr[1].text not in scope.macro_parameter_namespace:
-                    raise throw_positioned_syntax_error(
-                        scope,
-                        expr,
-                        "Could not find name in macro parameter space",
-                    )
-
-                value = scope.macro_parameter_namespace[expr[1].text]
-
-                if isinstance(value, ConstantAccessExpression):
-                    if not isinstance(value.value, str):
-                        raise throw_positioned_syntax_error(
-                            scope,
-                            expr,
-                            f"Expected 'string' for name de-referencing, got {value}",
-                        )
-
-                    return value.value
-
-                if isinstance(
-                    value,
-                    (
-                        GlobalAccessExpression,
-                        LocalAccessExpression,
-                        DerefAccessExpression,
-                    ),
-                ):
-                    return value.get_name(scope)
-
-                raise throw_positioned_syntax_error(
-                    scope,
-                    expr,
-                    f"Expected <static evaluated expression> for getting the name for storing, got {value}",
-                )
-
-            return get
+            return MacroExpandedIdentifier(expr[1].text, expr)
 
         if expr := self.try_consume(IdentifierToken):
-            return lambda scope: expr.text
+            return StaticIdentifier(expr.text)
 
-    def parse_identifier_like(
-        self, scope: ParsingScope
-    ) -> typing.Callable[[ParsingScope], str]:
+    def parse_identifier_like(self, scope: ParsingScope) -> IIdentifierAccessor:
         identifier = self.try_parse_identifier_like()
 
         if identifier is None:
