@@ -1,5 +1,7 @@
 import typing
 
+from bytecodemanipulation.Opcodes import Opcodes
+
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.MutableFunction import Instruction
@@ -9,8 +11,9 @@ from bytecodemanipulation.MutableFunction import MutableFunction
 class TopOfStackAccessExpression(AbstractAccessExpression):
     PREFIX = "%"
 
-    def __init__(self, token=None):
+    def __init__(self, token=None, offset=0):
         self.token = token
+        self.offset = offset
 
     def __eq__(self, other):
         return type(self) == type(other)
@@ -24,9 +27,22 @@ class TopOfStackAccessExpression(AbstractAccessExpression):
     def emit_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
+        if self.offset != 0:
+            return [
+                Instruction(function, -1, Opcodes.ROT_N, arg=self.offset)
+                for _ in range(self.offset - 1)
+            ] + [
+                Instruction(function, -1, Opcodes.DUP_TOP),
+                Instruction(function, -1, Opcodes.ROT_TWO),
+                Instruction(function, -1, Opcodes.ROT_N, arg=self.offset),
+            ]
+
         return []
 
     def emit_store_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
+        if self.offset != 0:
+            raise NotImplementedError("%<n> as store target")
+
         return []
