@@ -1,3 +1,4 @@
+import dis
 import functools
 import itertools
 from unittest import TestCase
@@ -12,7 +13,9 @@ from bytecodemanipulation.data.shared.instructions.FunctionDefinitionAssembly im
     AbstractFunctionDefinitionAssembly,
 )
 from bytecodemanipulation.data.shared.instructions.IfAssembly import AbstractIFAssembly
-from bytecodemanipulation.data.shared.instructions.JumpAssembly import AbstractJumpAssembly
+from bytecodemanipulation.data.shared.instructions.JumpAssembly import (
+    AbstractJumpAssembly,
+)
 from bytecodemanipulation.data.shared.instructions.LoadAssembly import LoadAssembly
 from bytecodemanipulation.data.shared.instructions.LoadConstAssembly import (
     AbstractLoadConstAssembly,
@@ -26,7 +29,9 @@ from bytecodemanipulation.data.shared.instructions.LoadGlobalAssembly import (
 from bytecodemanipulation.data.shared.instructions.PopElementAssembly import (
     AbstractPopElementAssembly,
 )
-from bytecodemanipulation.data.shared.instructions.ReturnAssembly import AbstractReturnAssembly
+from bytecodemanipulation.data.shared.instructions.ReturnAssembly import (
+    AbstractReturnAssembly,
+)
 from bytecodemanipulation.data.shared.instructions.StoreAssembly import StoreAssembly
 from bytecodemanipulation.data.shared.instructions.StoreFastAssembly import (
     AbstractStoreFastAssembly,
@@ -34,10 +39,16 @@ from bytecodemanipulation.data.shared.instructions.StoreFastAssembly import (
 from bytecodemanipulation.data.shared.instructions.StoreGlobalAssembly import (
     AbstractStoreGlobalAssembly,
 )
-from bytecodemanipulation.data.shared.instructions.WhileAssembly import AbstractWhileAssembly
-from bytecodemanipulation.data.shared.instructions.YieldAssembly import AbstractYieldAssembly
+from bytecodemanipulation.data.shared.instructions.WhileAssembly import (
+    AbstractWhileAssembly,
+)
+from bytecodemanipulation.data.shared.instructions.YieldAssembly import (
+    AbstractYieldAssembly,
+)
 from bytecodemanipulation.data.shared.instructions.OpAssembly import AbstractOpAssembly
-from bytecodemanipulation.data.shared.instructions.CallAssembly import AbstractCallAssembly
+from bytecodemanipulation.data.shared.instructions.CallAssembly import (
+    AbstractCallAssembly,
+)
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 bytecodemanipulation.data_loader.INIT_ASSEMBLY = False
@@ -62,7 +73,7 @@ bytecodemanipulation.data_loader.load_assembly_instructions()
 if typing.TYPE_CHECKING:
     pass
 
-from tests.test_issues import compare_optimized_results
+from tests.util import compare_optimized_results
 from bytecodemanipulation.Optimiser import cache_global_name, _OptimisationContainer
 
 
@@ -76,6 +87,9 @@ else:
     )
 
 
+from bytecodemanipulation.assembler.target import apply_operations
+
+
 GLOBAL = None
 
 
@@ -120,7 +134,9 @@ LOAD 19 --> $test
                     AbstractLoadGlobalAssembly.IMPLEMENTATION(10),
                     AbstractLoadGlobalAssembly.IMPLEMENTATION("test"),
                     AbstractLoadGlobalAssembly.IMPLEMENTATION(10),
-                    AbstractLoadGlobalAssembly.IMPLEMENTATION("hello", LocalAccessExpression("test")),
+                    AbstractLoadGlobalAssembly.IMPLEMENTATION(
+                        "hello", LocalAccessExpression("test")
+                    ),
                 ]
             ),
             expr,
@@ -144,7 +160,9 @@ LOAD 19 --> $test
                         "test",
                         LocalAccessExpression("test"),
                     ),
-                    AbstractStoreGlobalAssembly.IMPLEMENTATION("test", TopOfStackAccessExpression()),
+                    AbstractStoreGlobalAssembly.IMPLEMENTATION(
+                        "test", TopOfStackAccessExpression()
+                    ),
                 ]
             ),
             expr,
@@ -162,7 +180,9 @@ LOAD 19 --> $test
                     AbstractLoadFastAssembly.IMPLEMENTATION(10),
                     AbstractLoadFastAssembly.IMPLEMENTATION("test"),
                     AbstractLoadFastAssembly.IMPLEMENTATION(10),
-                    AbstractLoadFastAssembly.IMPLEMENTATION("test", GlobalAccessExpression("test")),
+                    AbstractLoadFastAssembly.IMPLEMENTATION(
+                        "test", GlobalAccessExpression("test")
+                    ),
                 ]
             ),
             expr,
@@ -178,9 +198,15 @@ LOAD 19 --> $test
                 [
                     AbstractStoreFastAssembly.IMPLEMENTATION("test"),
                     AbstractStoreFastAssembly.IMPLEMENTATION("test"),
-                    AbstractStoreFastAssembly.IMPLEMENTATION("test", GlobalAccessExpression("test")),
-                    AbstractStoreFastAssembly.IMPLEMENTATION("test", LocalAccessExpression("test")),
-                    AbstractStoreFastAssembly.IMPLEMENTATION("test", TopOfStackAccessExpression()),
+                    AbstractStoreFastAssembly.IMPLEMENTATION(
+                        "test", GlobalAccessExpression("test")
+                    ),
+                    AbstractStoreFastAssembly.IMPLEMENTATION(
+                        "test", LocalAccessExpression("test")
+                    ),
+                    AbstractStoreFastAssembly.IMPLEMENTATION(
+                        "test", TopOfStackAccessExpression()
+                    ),
                 ]
             ),
             expr,
@@ -196,8 +222,12 @@ LOAD 19 --> $test
                 [
                     AbstractLoadConstAssembly.IMPLEMENTATION(10),
                     AbstractLoadConstAssembly.IMPLEMENTATION("Hello World!"),
-                    AbstractLoadConstAssembly.IMPLEMENTATION(10, GlobalAccessExpression("test")),
-                    AbstractLoadConstAssembly.IMPLEMENTATION(GlobalAccessExpression("global")),
+                    AbstractLoadConstAssembly.IMPLEMENTATION(
+                        10, GlobalAccessExpression("test")
+                    ),
+                    AbstractLoadConstAssembly.IMPLEMENTATION(
+                        GlobalAccessExpression("global")
+                    ),
                     AbstractLoadConstAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("global"), LocalAccessExpression("test")
                     ),
@@ -307,17 +337,27 @@ CALL @test.x (*$x, @b, *%)
                 [
                     AbstractCallAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("print"),
-                        [AbstractCallAssembly.Arg(ConstantAccessExpression("Hello World!"))],
+                        [
+                            AbstractCallAssembly.Arg(
+                                ConstantAccessExpression("Hello World!")
+                            )
+                        ],
                     ),
                     AbstractCallAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("print"),
-                        [AbstractCallAssembly.Arg(ConstantAccessExpression("Hello World!"))],
+                        [
+                            AbstractCallAssembly.Arg(
+                                ConstantAccessExpression("Hello World!")
+                            )
+                        ],
                         LocalAccessExpression("result"),
                     ),
                     AbstractCallAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("print"),
                         [
-                            AbstractCallAssembly.Arg(ConstantAccessExpression("Hello World!")),
+                            AbstractCallAssembly.Arg(
+                                ConstantAccessExpression("Hello World!")
+                            ),
                             AbstractCallAssembly.Arg(ConstantAccessExpression("test")),
                         ],
                         LocalAccessExpression("result"),
@@ -335,14 +375,20 @@ CALL @test.x (*$x, @b, *%)
                     ),
                     AbstractCallAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("test"),
-                        [AbstractCallAssembly.KwArg("key", GlobalAccessExpression("value"))],
+                        [
+                            AbstractCallAssembly.KwArg(
+                                "key", GlobalAccessExpression("value")
+                            )
+                        ],
                         GlobalAccessExpression("result"),
                     ),
                     AbstractCallAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("test"),
                         [
                             AbstractCallAssembly.Arg(LocalAccessExpression("direct")),
-                            AbstractCallAssembly.KwArg("key", GlobalAccessExpression("value")),
+                            AbstractCallAssembly.KwArg(
+                                "key", GlobalAccessExpression("value")
+                            ),
                         ],
                         GlobalAccessExpression("result"),
                     ),
@@ -354,7 +400,9 @@ CALL @test.x (*$x, @b, *%)
                                     LocalAccessExpression("direct"), "test"
                                 )
                             ),
-                            AbstractCallAssembly.KwArg("key", GlobalAccessExpression("value")),
+                            AbstractCallAssembly.KwArg(
+                                "key", GlobalAccessExpression("value")
+                            ),
                         ],
                         GlobalAccessExpression("result"),
                     ),
@@ -385,6 +433,7 @@ OP @lhs[$local.attr] ** @rhs"""
                         AbstractOpAssembly.BinaryOperation(
                             GlobalAccessExpression("lhs"),
                             "+",
+                            [SpecialToken("+")],
                             GlobalAccessExpression("rhs"),
                         ),
                         GlobalAccessExpression("result"),
@@ -398,6 +447,7 @@ OP @lhs[$local.attr] ** @rhs"""
                                 ),
                             ),
                             "**",
+                            [SpecialToken("*"), SpecialToken("*")],
                             GlobalAccessExpression("rhs"),
                         )
                     ),
@@ -426,7 +476,9 @@ OP @lhs[$local.attr] ** @rhs"""
             CompoundExpression(
                 [
                     AbstractReturnAssembly.IMPLEMENTATION(),
-                    AbstractReturnAssembly.IMPLEMENTATION(GlobalAccessExpression("global")),
+                    AbstractReturnAssembly.IMPLEMENTATION(
+                        GlobalAccessExpression("global")
+                    ),
                 ]
             ),
             expr,
@@ -441,10 +493,16 @@ OP @lhs[$local.attr] ** @rhs"""
             CompoundExpression(
                 [
                     AbstractYieldAssembly.IMPLEMENTATION(),
-                    AbstractYieldAssembly.IMPLEMENTATION(GlobalAccessExpression("global")),
+                    AbstractYieldAssembly.IMPLEMENTATION(
+                        GlobalAccessExpression("global")
+                    ),
                     AbstractYieldAssembly.IMPLEMENTATION(is_star=True),
-                    AbstractYieldAssembly.IMPLEMENTATION(LocalAccessExpression("local"), is_star=True),
-                    AbstractYieldAssembly.IMPLEMENTATION(target=TopOfStackAccessExpression()),
+                    AbstractYieldAssembly.IMPLEMENTATION(
+                        LocalAccessExpression("local"), is_star=True
+                    ),
+                    AbstractYieldAssembly.IMPLEMENTATION(
+                        target=TopOfStackAccessExpression()
+                    ),
                     AbstractYieldAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("global"),
                         target=LocalAccessExpression("local"),
@@ -501,6 +559,7 @@ IF @global 'test'
                             AbstractOpAssembly.BinaryOperation(
                                 LocalAccessExpression("a"),
                                 "==",
+                                [SpecialToken("="), SpecialToken("=")],
                                 LocalAccessExpression("b"),
                             )
                         ),
@@ -510,7 +569,9 @@ IF @global 'test'
                     ),
                     AbstractIFAssembly.IMPLEMENTATION(
                         GlobalAccessExpression("global"),
-                        CompoundExpression([AbstractJumpAssembly.IMPLEMENTATION("test")]),
+                        CompoundExpression(
+                            [AbstractJumpAssembly.IMPLEMENTATION("test")]
+                        ),
                         "test",
                     ),
                 ]
@@ -556,6 +617,7 @@ WHILE $local 'test' {
                             AbstractOpAssembly.BinaryOperation(
                                 LocalAccessExpression("a"),
                                 "==",
+                                [SpecialToken("="), SpecialToken("=")],
                                 LocalAccessExpression("b"),
                             )
                         ),
@@ -585,24 +647,30 @@ WHILE $local 'test' {
                 [
                     LabelAssembly("test"),
                     AbstractJumpAssembly.IMPLEMENTATION("test"),
-                    AbstractJumpAssembly.IMPLEMENTATION("test", GlobalAccessExpression("global")),
+                    AbstractJumpAssembly.IMPLEMENTATION(
+                        "test", GlobalAccessExpression("global")
+                    ),
                     AbstractJumpAssembly.IMPLEMENTATION(
                         "test",
                         AbstractOpAssembly.IMPLEMENTATION(
                             AbstractOpAssembly.BinaryOperation(
                                 GlobalAccessExpression("a"),
                                 "==",
+                                [SpecialToken("="), SpecialToken("=")],
                                 GlobalAccessExpression("b"),
                             )
                         ),
                     ),
-                    AbstractJumpAssembly.IMPLEMENTATION("test", GlobalAccessExpression("global")),
+                    AbstractJumpAssembly.IMPLEMENTATION(
+                        "test", GlobalAccessExpression("global")
+                    ),
                     AbstractJumpAssembly.IMPLEMENTATION(
                         "test",
                         AbstractOpAssembly.IMPLEMENTATION(
                             AbstractOpAssembly.BinaryOperation(
                                 GlobalAccessExpression("a"),
                                 "==",
+                                [SpecialToken("="), SpecialToken("=")],
                                 GlobalAccessExpression("b"),
                             )
                         ),
@@ -613,6 +681,7 @@ WHILE $local 'test' {
                             AbstractOpAssembly.BinaryOperation(
                                 GlobalAccessExpression("a"),
                                 "==",
+                                [SpecialToken("="), SpecialToken("=")],
                                 GlobalAccessExpression("b"),
                             )
                         ),
@@ -639,7 +708,9 @@ DEF test <test> () -> @target {}
         self.assertEqualList(
             CompoundExpression(
                 [
-                    AbstractFunctionDefinitionAssembly.IMPLEMENTATION("test", [], [], CompoundExpression([])),
+                    AbstractFunctionDefinitionAssembly.IMPLEMENTATION(
+                        "test", [], [], CompoundExpression([])
+                    ),
                     AbstractFunctionDefinitionAssembly.IMPLEMENTATION(
                         "test", ["test"], [], CompoundExpression([])
                     ),
@@ -904,26 +975,6 @@ PYTHON {
 
         bytecodemanipulation.assembler.hook.unhook()
 
-    def test_walrus_assigment(self):
-        def target():
-            assembly(
-                """
-OP $local := 10
-STORE $test
-RETURN OP ($local + $test)
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        def compare():
-            test = (local := 10)
-            return local + test
-
-        compare_optimized_results(self, target, compare, opt_ideal=0)
-
     def test_dynamic_attribute_access(self):
         def target(t):
             assembly(
@@ -938,110 +989,6 @@ RETURN $t.("test_dynamic_attribute_access")
 
         self.assertEqual(target(self), self.test_dynamic_attribute_access)
 
-    def test_isinstance_check(self):
-        def target():
-            assembly(
-                """
-RETURN OP (10 isinstance @int)
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        self.assertTrue(target())
-
-    def test_not_isinstance_check(self):
-        def target():
-            assembly(
-                """
-RETURN OP (10 isinstance @float)
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        # dis.dis(target)
-
-        self.assertFalse(target())
-
-    def test_issubclass_check(self):
-        def target():
-            assembly(
-                """
-RETURN OP (@int issubclass @object)
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        self.assertTrue(target())
-
-    def test_not_issubclass_check(self):
-        def target():
-            assembly(
-                """
-RETURN OP (@int issubclass @float)
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        # dis.dis(target)
-
-        self.assertFalse(target())
-
-    def test_hasattr_check(self):
-        def target(t):
-            assembly(
-                """
-RETURN OP ($t hasattr "test_hasattr_check")
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        self.assertTrue(target(self))
-
-    def test_not_hasattr_check(self):
-        def target(t):
-            assembly(
-                """
-RETURN OP ($t hasattr "no attr")
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        # dis.dis(target)
-
-        self.assertFalse(target(self))
-
-    def test_getattr_check(self):
-        def target(t):
-            assembly(
-                """
-RETURN OP ($t getattr "test_getattr_check")
-"""
-            )
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        self.assertEqual(target(self), self.test_getattr_check)
-
 
 class TestMacro(TestCase):
     def test_basic(self):
@@ -1489,7 +1436,7 @@ CLASS test
         def target():
             assembly(
                 """
-CLASS test
+CLASS test<test>
 {
     MACRO test_macro()
     {
@@ -1650,6 +1597,8 @@ FOREACH $p IN $iterable
         apply_inline_assemblies(mutable)
         mutable.reassign_to_function()
 
+        # dis.dis(target)
+
         self.assertEqual(target(), [1, 2, 3, 4])
 
     def test_for_loop_double(self):
@@ -1693,3 +1642,68 @@ FOREACH $p IN $iterable * $iterable_2
         mutable.reassign_to_function()
 
         self.assertEqual(target(), [0, 0, 1, 2])
+
+    def test_raw_assembly_by_id(self):
+        def target():
+            assembly(
+                """
+LOAD 10
+LOAD 5
+RAW 2
+RETURN %
+"""
+            )
+
+        mutable = MutableFunction(target)
+        apply_inline_assemblies(mutable)
+        mutable.reassign_to_function()
+
+        self.assertEqual(target(), 10)
+
+    def test_raw_assembly_by_name(self):
+        def target():
+            assembly(
+                """
+LOAD 10
+LOAD 5
+RAW ROT_TWO
+RETURN %
+"""
+            )
+
+        mutable = MutableFunction(target)
+        apply_inline_assemblies(mutable)
+        mutable.reassign_to_function()
+
+        self.assertEqual(target(), 10)
+
+    def test_assert(self):
+        @apply_operations
+        def target(x):
+            assembly(
+                """
+ASSERT $x
+"""
+            )
+
+        target(1)
+        self.assertRaises(AssertionError, lambda: target(0))
+
+    def test_assert_with_message(self):
+        @apply_operations
+        def target(x):
+            assembly(
+                """
+ASSERT $x \"Test Message\"
+"""
+            )
+
+        target(1)
+        self.assertRaises(AssertionError, lambda: target(0))
+
+        try:
+            target(0)
+        except AssertionError as e:
+            self.assertEqual(e.args, ("Test Message",))
+        else:
+            self.assertTrue(False)
